import inspect
import os.path
from datetime import datetime as dt, timedelta, timezone


class LoggerUtil:

    def __init__(self,
                 logdir_abspath: str,
                 log_ext: str = "log",
                 use_timestamp: bool = True):

        self.logdir_path = logdir_abspath
        if not os.path.exists(self.logdir_path):
            os.mkdir(self.logdir_path)

        self.log_ext = log_ext
        self.use_timestamp = use_timestamp

    def log_info(self, filename: str, message: str = '',
                 visible_tag: bool = True, stdout_target: bool = False):
        self._log_write_to_file(filename, message,
                                visible_tag, stdout_target)

    def log_debug(self, filename: str, message: str = '',
                  visible_tag: bool = True, stdout_target: bool = False):
        self._log_write_to_file(filename, message,
                                visible_tag, stdout_target)

    def log_warn(self, filename: str, message: str = '',
                 visible_tag: bool = True, stdout_target: bool = False):
        self._log_write_to_file(filename, message,
                                visible_tag, stdout_target)

    def log_error(self, filename: str, message: str = '',
                  visible_tag: bool = True, stdout_target: bool = False):
        self._log_write_to_file(filename, message,
                                visible_tag, stdout_target)

    def log_fatal(self, filename: str, message: str = '',
                  visible_tag: bool = True, stdout_target: bool = False):
        self._log_write_to_file(filename, message,
                                visible_tag, stdout_target)

    def _log_write_to_file(self, filename: str, message: str = '',
                           visible_tag: bool = True, stdout_target: bool = True):
        constr = list()
        if self.use_timestamp:
            constr.append(
                "[{}]".format(
                    dt.now(
                        timezone(timedelta(hours=9), name="KST")
                    ).strftime("%Y-%m-%d %H:%M:%S.%f %Z")
                )
            )

        constr.append("[{} - {}]".format(filename, inspect.stack()[1].function.upper()))

        if not visible_tag:
            constr = list()

        constr.append("{}\n".format(message))

        if stdout_target:
            print(' '.join(constr))
        else:
            file_fullpath = os.path.join(
                self.logdir_path,
                '.'.join(
                    [filename, self.log_ext]
                )
            )

            lf = open(file_fullpath, "a+") \
                if os.path.exists(file_fullpath) \
                else open(file_fullpath, "w")

            lf.write(' '.join(constr))
            lf.close()

