import math
import pandas as pd


class RefineData:

    @staticmethod
    def k_val_str(a: int | str) -> str:
        return str(a) if (type(a).__name__ == 'int'
                          or type(a).__name__ == 'int64') else \
                str(int(a)) if (type(a).__name__ == 'float'
                                or type(a).__name__ == 'float64') else a

    @staticmethod
    def k_val_int(a: int | str) -> int:
        return int(a) if type(a).__name__ == 'str' or (
            type(a).__name__ == 'float' or type(a).__name__ == 'float64'
        ) else a

    @staticmethod
    def k_val_float(a: int | float | str) -> float:
        return float(a) if type(a).__name__ in ['str', 'int', 'int64'] else a

    # 1
    @staticmethod
    def bool_val(a: str | int, offset: int = 0) -> bool:
        if type(a).__name__ == 'int' or type(a).__name__ == 'int64':
            return False if a >= 66666 else bool(a - offset)
        elif type(a).__name__ == 'str':
            try:
                n = int(a)
                return False if n >= 66666 else bool(n - offset)
            except ValueError:
                return False

    # 2
    @staticmethod
    def get_age_weight(a: int | str) -> int:
        try:
            a = RefineData.k_val_int(a)

            if a < 40:
                return a if 1 <= a <= 5 else 1
            elif a >= 70:
                return 5 if a < 200 else 0
            else:
                return int((a - 30) // 10)
        except ValueError:
            return 0

    # 3
    @staticmethod
    def get_treated_val(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["cu_end", "cu_ing", "been_cu", "nvr_cu"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 4
    @staticmethod
    def freq_unit_val(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["year", "month", "week"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 5
    @staticmethod
    def family_rel_val(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Father", "Mother", "Brother",
                      "Sister", "Son", "Daughter"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 6
    @staticmethod
    def get_cancer_name(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["STAD", "HCC", "CRC", "BRCA", "CESC",
                      "BAC", "THCA", "PRAD", "BLCA", "etc."]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return a  # as is.

    # 7
    @staticmethod
    def neg_bool_val(a: str | int, offset: int = 0) -> bool:
        return not(RefineData.bool_val(a, offset))

    # 8
    @staticmethod
    def inhale_freq_week(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.0, 1.5, 3.5, 5.5, 7.0]
            return v_list[k] if 0 <= k < len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 9
    @staticmethod
    def intake_freq_month(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.0, 1.0, 2.5, 4.5, 10.0, 20.0, 30.0, 60.0]
            return v_list[k] if 0 <= k < len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 10
    @staticmethod
    def intake_term_year(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.0, 2.5, 7.5, 15.0, 25.0, 35.0, 45.0]
            return v_list[k - 1] if 0 <= k < len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 11
    @staticmethod
    def inhale_depth(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["hardly", "a little", "normally", "deeply"]
            return v_list[k] if 0 <= k < len(v_list) else ""
        except ValueError:
            return ""

    # 12: 17 + 2 * (n - 1)
    @staticmethod
    def val_12(a: int | str) -> int:
        try:
            k = RefineData.k_val_int(a)
            return 17 + 2 * (k - 1) if k < 66666 else 0
        except ValueError:
            return 0

    # 13
    @staticmethod
    def fat_list(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["soy", "corn", "olive", "sesame",
                      "perilla", "butter", "margarine", "etc"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 14
    @staticmethod
    def intake_freq_week_day_unit(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["week", "day"]
            return v_list[k - 1] if k in [1, 2] else ""
        except ValueError:
            return ""

    # 15
    @staticmethod
    def intake_freq_week_day(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.0, 1.0, 2.5, 4.5, 7.0, 14.0, 21.0]
            return v_list[k - 1] if 0 <= k < len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 16
    @staticmethod
    def beef_roasting_level(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["No Intake", "WellDone", "Medium", "Rare"]
            return v_list[k % 4] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 17
    @staticmethod
    def burnt_beef_intake(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Nothing", "Sometimes", "AlotOfTime"]
            return v_list[k - 1] if 0 <= k < len(v_list) else ""
        except ValueError:
            return ""

    # 18
    @staticmethod
    def cattle_fat_intake(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["DoNotCare", "CutAlmostOff", "CutAllOff"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 19
    @staticmethod
    def number_of_taking_meal(a: int | str) -> int:
        try:
            k = RefineData.k_val_int(a)
            if k > 5 or k <= 0:
                return 0
            else:
                return -1 if k == 5 else k
        except ValueError:
            return 0

    # 20
    @staticmethod
    def intake_freq_week(a: int | str) -> int:
        try:
            k = RefineData.k_val_int(a)
            if k < 5:
                return 0 if k == 0 else k * 2 - 1
            else:
                return 0
        except ValueError:
            return 0

    # 21
    @staticmethod
    def sort_of_meal(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["rice", "multigrain", "both"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 22
    @staticmethod
    def sort_of_mulgrn_meal(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["beanMixed", "anotherGrainMixed"]
            return v_list[k - 1] if k in [1, 2] else ""
        except ValueError:
            return ""

    # 23, 24
    @staticmethod
    def y_ax_float(x: int | float | str, a: float = 1.0) -> float:
        try:
            k = RefineData.k_val_int(x)
            return float(a * k) if k != 0 and k < 66666 else 0.0
        except ValueError:
            try:
                k = RefineData.k_val_float(x)
                return float(a * k) if k != 0.0 and k < 66666.0 else 0.0
            except ValueError:
                return 0.0

    # 25, 26, 32, 38
    @staticmethod
    def y_ax_int(x: int | str, a: int = 1) -> int:
        try:
            k = RefineData.k_val_int(x)
            return int(a * k) if k != 0 and k < 66666 else 0
        except ValueError:
            return 0

    # 27
    @staticmethod
    def inc_val(a: int | str) -> int:
        try:
            k = RefineData.k_val_int(a)
            return k + 1 if k < 66666 else 0
        except ValueError:
            return 0

    # 28, 29
    @staticmethod
    def inc_exp_from_fval(x: int | float | str, a: float = 1.0) -> float:
        try:
            k = RefineData.k_val_int(x)
            return 0.0 if k == 0 or k >= 66666 else a * (2 ** (k - 1))
        except ValueError:
            try:
                k = RefineData.k_val_float(x)
                return 0.0 if k == 0 or k >= 66666.0 else a * (2 ** (k - 1))
            except ValueError:
                return 0.0

    # 30-1
    @staticmethod
    def fib(a: int) -> int:
        return 1 if a <= 2 else \
            RefineData.fib(a - 1) \
            + RefineData.fib(a - 2)

    # 30
    @staticmethod
    def get_fib_seq(a: int | str, offset: int = 0) -> int:
        try:
            k = RefineData.k_val_int(a)
            return RefineData.fib(k + offset) if k < 66666 else 0
        except ValueError:
            return 0

    # 31, 33
    @staticmethod
    def get_exp_num_with_n_times(a: int | str, mult: int = 1) -> int:
        try:
            k = RefineData.k_val_int(a)
            return 2 ** (k - 1) * mult if k < 66666 else 0
        except ValueError:
            return 0

    # 34
    @staticmethod
    def get_val34(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.125, 0.25, 0.333]
            return v_list[k - 1] if 1 <= k <= len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 35
    @staticmethod
    def get_val35(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.25, 0.3333, 0.5]
            return v_list[k - 1] if 1 <= k <= len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 36
    @staticmethod
    def food_intake_freq_month(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0, 1.0, 2.5, 6.0, 14.0, 22.0, 30.0, 60.0, 90.0]
            return v_list[k] if 0 <= k < len(v_list) else 0.0

        except ValueError:
            return 0.0

    # 37
    @staticmethod
    def drink_intake_freq_month(a: int | str) -> float:
        try:
            k = RefineData.k_val_int(a)
            v_list = [0.0, 1.0, 2.5, 6.0, 14.0, 22.0, 45.0, 105.0, 150.0]
            return v_list[k - 1] if 1 <= k <= len(v_list) else 0.0
        except ValueError:
            return 0.0

    # 39
    @staticmethod
    def get_urine_react_value(a: int | str, offset: int = 0) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Negative", "Trace", "1 Positive",
                      "2 Positive", "3 Positive", "4 Positive"]
            return v_list[k - offset] if offset <= k < len(v_list) - offset else ""
        except ValueError:
            return ""

    # 40
    @staticmethod
    def get_liquor_glass_size(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Bowl", "Tiny", "Wine", "PintPot", "OnTheRock"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 41
    @staticmethod
    def get_urine_exm_others(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["None", "Yeast like organism", "Spermatozoa"]
            return v_list[k] if 0 <= k < len(v_list) else ""
        except ValueError:
            return ""

    # 42
    @staticmethod
    def get_ldl_val(tchl: int | float, hdl: int | float,
                    tg: int | float) -> float:
        return tchl - hdl - (0.2 * tg)

    # 43
    @staticmethod
    def get_ecg_result(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Normal", "Otherwise Normal", "Borderline", "Abnormal"]
            return v_list[k - 1] if 1 <= k <= len(v_list) else ""
        except ValueError:
            return ""

    # 44
    @staticmethod
    def get_ecg_ekg1_result(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Normal", "Borderline", "Abnormal"]
            return v_list[k] if 0 <= k < len(v_list) else ""
        except ValueError:
            return ""

    # 45
    @staticmethod
    def get_ecg_ekg2_result(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Normal", "Abnormal", "Not Measured"]
            return v_list[k] if 0 <= k < len(v_list) else ""
        except ValueError:
            return ""

    # 46
    @staticmethod
    def get_gender_string(a: int | str) -> str:
        try:
            k = RefineData.k_val_int(a)
            v_list = ["Female", "Male"]
            return v_list[k - 1] if k in [1, 2] else ""
        except ValueError:
            return ""

    # noinspection PyBroadException
    @staticmethod
    def calibrate_data_type(a: int | str | float | bool, data_type: str):
        if data_type == 'str':
            if pd.isna(a):
                return ''
            elif a is not None:
                if type(a).__name__ == 'str' and a in ['66666', '77777', '99999']:
                    return ''
                elif type(a).__name__ in ['int', 'int64', 'float', 'float64']:
                    return '' if (a in [66666, 77777, 99999]
                                  or a in [66666.0, 77777.0, 99999.0]) \
                        else str(a)
                elif not a.isascii():
                    ar = ''
                    dencode_pair = [
                        ['utf-8', 'utf-8'],
                        ['iso-8859-1', 'cp949'],
                        ['iso-8859-1', 'euc-kr']
                    ]
                    i = 0
                    alen = len(bytes(a, dencode_pair[i][0]))
                    while True:
                        try:
                            ar = bytes(a, dencode_pair[i][0])[:alen].decode(dencode_pair[i][1])
                            if ar == '':
                                if i == len(dencode_pair) - 1:
                                    alen -= 1
                                    i = 0
                                else:
                                    i += 1
                                continue
                            break
                        except Exception:
                            if i == len(dencode_pair) - 1:
                                alen -= 1
                                i = 0
                            else:
                                i += 1
                            continue

                    return ar
                else:
                    return str(a) if type(a).__name__ != data_type else a
            else:
                return ''
        elif data_type == 'int':
            if pd.isna(a):
                return 0
            else:
                if type(a).__name__ == 'str':
                    try:
                        n = int(a) \
                            if a not in ['66666', '77777', '99999'] \
                            else 0
                    except ValueError:
                        return 0

                    return n
                elif type(a).__name__ == 'float' \
                        or type(a).__name__ == 'float64':
                    return int(math.floor(a)) \
                        if a not in [66666.0, 77777.0, 99999.0] \
                        else 0
                elif a is None:
                    return 0
                else:
                    return int(a) if a not in [66666, 77777, 99999] else 0
        elif data_type == 'float':
            if pd.isna(a):
                return 0.0
            else:
                if type(a).__name__ == 'str':
                    try:
                        n = float(a) if a not in ['66666', '77777', '99999'] else 0.0
                    except ValueError:
                        return 0.0

                    return n
                elif a is None:
                    return 0.0
                else:
                    return float(a)
        elif data_type == 'bool':
            if pd.isna(a):
                return 0
            else:
                if type(a).__name__ == 'str':
                    try:
                        n = bool(a) if a not in ['66666', '77777', '99999'] else False
                        return 1 if n else 0
                    except ValueError:
                        return 0
                elif type(a).__name__ in ['int', 'int64', 'float', 'float64']:
                    return 1 if (a != 0 and a != 0.0) \
                        and a not in [66666, 77777, 99999] \
                        and a not in [66666.0, 77777.0, 99999.0] else 0
                elif type(a).__name__ == 'bool':
                    return 1 if a else 0
                else:
                    return False

    @staticmethod
    def get_refined_value(vcode: int, a: int | str, **kwargs):

        if vcode == 0:
            if type(a).__name__ == 'int' or type(a).__name__ == 'int64':
                return a if a < 66666 else 0
            elif type(a).__name__ == 'float' or type(a).__name__ == 'float64':
                return a if a < 66666.0 else 0.0
            elif type(a).__name__ == 'str':
                return a if (a not in [66666, 77777, 99999]
                             and a not in [66666.0, 77777.0, 99999.0]) \
                    else ''
        elif vcode == 1:
            return RefineData.bool_val(a, 1) if a is not None else False
        elif vcode == 2:
            return RefineData.get_age_weight(a)
        elif vcode == 3:
            return RefineData.get_treated_val(a)
        elif vcode == 4:
            return RefineData.freq_unit_val(a)
        elif vcode == 5:
            return RefineData.family_rel_val(a)
        elif vcode == 6:
            return RefineData.get_cancer_name(a)
        elif vcode == 7:
            return RefineData.neg_bool_val(a, 1)
        elif vcode == 8:
            return RefineData.inhale_freq_week(a)
        elif vcode == 9:
            return RefineData.intake_freq_month(a)
        elif vcode == 10:
            return RefineData.intake_term_year(a)
        elif vcode == 11:
            return RefineData.inhale_depth(a)
        elif vcode == 12:
            return RefineData.val_12(a)
        elif vcode == 13:
            return RefineData.fat_list(a)
        elif vcode == 14:
            return RefineData.intake_freq_week_day_unit(a)
        elif vcode == 15:
            return RefineData.intake_freq_week_day(a)
        elif vcode == 16:
            return RefineData.beef_roasting_level(a)
        elif vcode == 17:
            return RefineData.burnt_beef_intake(a)
        elif vcode == 18:
            return RefineData.cattle_fat_intake(a)
        elif vcode == 19:
            return RefineData.number_of_taking_meal(a)
        elif vcode == 20:
            return RefineData.intake_freq_week(a)
        elif vcode == 21:
            return RefineData.sort_of_meal(a)
        elif vcode == 22:
            return RefineData.sort_of_mulgrn_meal(a)
        elif vcode in [23, 24]:
            m_list = [0.5, 0.3333]
            return RefineData.y_ax_float(a, m_list[vcode - 23])
        elif vcode in [25, 26, 32, 38]:
            m_dict = {"25": 2, "26": 3, "32": 5, "38": 4}
            return RefineData.y_ax_int(a, m_dict[str(vcode)])
        elif vcode == 27:
            return RefineData.inc_val(a)
        elif vcode in [28, 29]:
            m_list = [0.5, 0.25]
            return RefineData.inc_exp_from_fval(a, m_list[vcode - 28])
        elif vcode == 30:
            return RefineData.get_fib_seq(a, 3)
        elif vcode in [31, 33]:
            mult = int(not(bool(vcode - 31))) * 100
            mult = 1 if mult == 0 else mult
            return RefineData.get_exp_num_with_n_times(a, mult)
        elif vcode == 34:
            return RefineData.get_val34(a)
        elif vcode == 35:
            return RefineData.get_val35(a)
        elif vcode == 36:
            return RefineData.food_intake_freq_month(a)
        elif vcode == 37:
            return RefineData.drink_intake_freq_month(a)
        elif vcode == 39:
            return RefineData.get_urine_react_value(a, kwargs['urine_offset'])
        elif vcode == 40:
            return RefineData.get_liquor_glass_size(a)
        elif vcode == 41:
            return RefineData.get_urine_exm_others(a)
        elif vcode == 42:
            return RefineData.get_ldl_val(kwargs['tchl'], kwargs['hdl'], kwargs['tg'])
        elif vcode == 43:
            return RefineData.get_ecg_result(a)
        elif vcode == 44:
            return RefineData.get_ecg_ekg1_result(a)
        elif vcode == 45:
            return RefineData.get_ecg_ekg2_result(a)
        elif vcode == 46:
            return RefineData.get_gender_string(a)
